package es.verdnatura.presentation.view.feature.ajustes.fragment

import android.app.AlertDialog
import android.os.Bundle
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.MobileApplication
import es.verdnatura.R
import es.verdnatura.databinding.FragmentAjustesBinding
import es.verdnatura.domain.ConstAndValues.ANDROID_ID
import es.verdnatura.domain.ConstAndValues.ITEMPACKING
import es.verdnatura.domain.ConstAndValues.MENUBYDEFAULTSELECTED
import es.verdnatura.domain.ConstAndValues.NUMBEROFWAGONS
import es.verdnatura.domain.ConstAndValues.PRINTERFK
import es.verdnatura.domain.ConstAndValues.PRINTERNAME
import es.verdnatura.domain.ConstAndValues.SECTORDESCRIP
import es.verdnatura.domain.ConstAndValues.SECTORFK
import es.verdnatura.domain.ConstAndValues.TRAINFK
import es.verdnatura.domain.ConstAndValues.TRAINNAME
import es.verdnatura.domain.ConstAndValues.WAREHOUSEFK
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnAjustesItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.adapter.SettingsAdapter
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.Printers
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.Train
import es.verdnatura.presentation.view.feature.articulo.model.ItemPackingType
import es.verdnatura.presentation.view.feature.login.fragment.LoginViewModel
import es.verdnatura.presentation.view.feature.pasillero.fragment.PasilleroViewModel
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import kotlinx.coroutines.runBlocking

class AjustesFragment :
    BaseFragment<FragmentAjustesBinding, AjustesViewModel>(AjustesViewModel::class) {

    private lateinit var customDialog: CustomDialog
    private var sectorListVO: List<SectorItemVO> = listOf()
    private var printersList: List<Printers> = listOf()
    private var trainsList: List<Train> = listOf()
    private var settingsAdapter: SettingsAdapter? = null
    private var isWorkerAllowed: Boolean = false
    private var itemPackingList: List<ItemPackingType> = listOf()
    private var messagePrinter: String? = null
    private lateinit var loginViewModel: LoginViewModel
    private lateinit var pasilleroViewModel: PasilleroViewModel

    companion object {
        fun newInstance() = AjustesFragment()
    }

    override fun getLayoutId(): Int = R.layout.fragment_ajustes

    override fun onCreate(savedInstanceState: Bundle?) {

        customDialog = CustomDialog(requireContext())
        super.onCreate(savedInstanceState)

    }

    override fun init() {

        binding.mainToolbar.toolbarTitle.text = getString(R.string.settings)
        hideBackButton(binding.mainToolbar)
        viewModel.workerIsF11Allowed()
        binding.userText.text = mobileApplication.userName
        binding.itemVersion.text = requireActivity().packageManager.getPackageInfo(
            requireActivity().packageName, 0
        ).versionName!!
        binding.androididText.text =
            mobileApplication.dataStoreApp.readDataStoreKey<String>(ANDROID_ID)
        binding.serialNumber.text = mobileApplication.serialNumber

        super.init()
    }

    override fun observeViewModel() {
        with(viewModel) {

            settingsAdapter = SettingsAdapter(settingsItem, object : OnAjustesItemClickListener {
                override fun onAjustesItemClickListener(item: AjustesItemVO) {
                    when (item.title) {
                        getString(R.string.sector) -> viewModel.getSectorsSalix()
                        getString(R.string.closeSession) -> ma.onMyBackPressed()
                        getString(R.string.printerLabel) -> {
                            messagePrinter = getString(R.string.SelectPrinter)
                            viewModel.printerGet(
                                mobileApplication.dataStoreApp.readDataStoreKey(
                                    SECTORFK
                                )
                            )
                        }

                        getString(R.string.vehicleControl) -> ma.onPasillerosItemClickListener(
                            PasillerosItemVO(
                                title = getString(R.string.titleUserControlVehicle)
                            ), mobileApplication.userId.toString()
                        )

                        getString(R.string.menuByDefault) -> showMenuByDefault()
                        getString(R.string.settingsTitleItemPacking) -> if (item.action) viewModel.getItemPackingTypeSalix()
                        getString(R.string.settingsTitleTrain) -> if (item.action) viewModel.trainGet()

                        else -> {}
                    }

                }
            })

            workerAllowedResponse.observe(viewLifecycleOwner) {

                isWorkerAllowed = it
                loginViewModel = LoginViewModel(context as MobileApplication)
                loginViewModel.operator_getDataSalix(mobileApplication.userId!!)
                loginViewModel.workerOperator.observe(this@AjustesFragment) { iti ->
                    if (!iti.isError) {
                        runBlocking { mobileApplication.dataStoreApp.saveDataOperator(iti) }
                    } else {
                        ma.messageWithSound(iti.errorMessage, isError = true, isPlayed = true)
                    }
                }

                setSettings(isWorkerAllowed)

            }

            loadActionUpdate.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull {
                    messagePrinter =
                        getString(R.string.SelectPrinter) + "\n" + getString(R.string.printerRemoved)
                    viewModel.printerGet(mobileApplication.dataStoreApp.readDataStoreKey(SECTORFK))
                }
            }

            binding.setttingsItems.adapter = settingsAdapter
            binding.setttingsItems.layoutManager =
                LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

            loadSectorList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull { it ->

                    if (it.list.isNotEmpty()) {
                        val listSectores: ArrayList<String> = ArrayList()
                        it.list.forEach {
                            listSectores.add(it.description)
                        }
                        // modificar
                        listSectores.sort()
                        val array = arrayOfNulls<String>(listSectores.size)
                        sectorListVO = it.list
                        showDialogForAll(
                            listSectores.toArray(array), getString(R.string.selectSector)
                        )

                    } else {
                        customDialog.setTitle(getString(R.string.sectors))
                            .setDescription(getString(R.string.Noexistessectores))
                            .setOkButton(getString(R.string.Close)) {
                                customDialog.dismiss()
                            }.show()
                    }

                }
            }
            loadItemPackingList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull { it ->

                    if (it.list.isNotEmpty()) {
                        val listItemsPacking: ArrayList<String> = ArrayList()
                        it.list.forEach {
                            listItemsPacking.add(it.description)
                        }
                        val array = arrayOfNulls<String>(listItemsPacking.size)
                        itemPackingList = it.list
                        showDialogForAll(
                            listItemsPacking.toArray(array),
                            getString(R.string.settingsTitleItemPacking)
                        )

                    } else {
                        customDialog.setTitle(getString(R.string.settingsTitleItemPacking))
                            .setDescription(getString(R.string.NoItemPackingType))
                            .setOkButton(getString(R.string.Close)) {
                                customDialog.dismiss()
                            }.show()
                    }

                }
            }
            loadPrintersList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull { it ->

                    if (it.list.isNotEmpty()) {
                        val listPrinters: ArrayList<String> = ArrayList()
                        it.list.forEach {
                            listPrinters.add(it.name)
                        }
                        val array = arrayOfNulls<String>(listPrinters.size)
                        printersList = it.list
                        showDialogForAll(listPrinters.toArray(array), messagePrinter ?: "")

                    } else {
                        viewModel.settingsItem[2].selected = getString(R.string.noprinter)
                        settingsAdapter!!.notifyItemChanged(2)
                        customDialog.setTitle(getString(R.string.printers))
                            .setDescription(getString(R.string.Noprinters))
                            .setOkButton(getString(R.string.Close)) {
                                customDialog.dismiss()
                            }.show()
                    }

                }
            }
            loadTrainList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull { it ->

                    if (it.list.isNotEmpty()) {
                        val listTrain: ArrayList<String> = ArrayList()
                        it.list.forEach {
                            listTrain.add(it.name)
                        }
                        val array = arrayOfNulls<String>(listTrain.size)
                        trainsList = it.list
                        showDialogForAll(
                            listTrain.toArray(array), getString(R.string.selectTrain)
                        )

                    }

                }
            }
        }
        super.observeViewModel()
    }

    private fun showMenuByDefault() {
        pasilleroViewModel = PasilleroViewModel(mobileApplication)
        pasilleroViewModel.inititializeDefaultDataInit()
        val arrayList: ArrayList<String> = arrayListOf()

        pasilleroViewModel.pasillerositem.forEach {
            arrayList.add(it.title)
        }
        arrayList.add(getString(R.string.all))
        arrayList.sorted()
        showDialogForAll((arrayList).toTypedArray(), getString(R.string.menuByDefault))
    }

    private fun setSettings(actionIsAllowed: Boolean = false) {

        runBlocking {

            viewModel.inititializeDefaultAjusts(
                mobileApplication.dataStoreApp.readDataStoreKey(SECTORDESCRIP),
                mobileApplication.dataStoreApp.readDataStoreKey(SECTORFK),
                (mobileApplication.dataStoreApp.readDataStoreKey(NUMBEROFWAGONS) as Int).toString(),
                mobileApplication.dataStoreApp.readDataStoreKey(PRINTERFK),
                mobileApplication.dataStoreApp.readDataStoreKey(PRINTERNAME),
                mobileApplication.dataStoreApp.readDataStoreKey<String>(TRAINNAME),
                mobileApplication.dataStoreApp.readDataStoreKey<String>(ITEMPACKING),
                actionIsAllowed = actionIsAllowed,
                menuDefault = mobileApplication.dataStoreApp.readDataStoreKey(
                    MENUBYDEFAULTSELECTED
                )

            )
        }
        binding.splashProgress.visibility = View.GONE
    }

    private fun showDialogForAll(array: Array<String>, title: String) {
        val builder = AlertDialog.Builder(this.context)
        builder.setTitle(title)

        builder.setItems(array) { _, which ->
            val selected = array[which]
            when (title) {
                getString(R.string.settingsTitleItemPacking) -> {
                    itemPackingList.forEach {
                        if (it.description == selected) {

                            runBlocking {
                                mobileApplication.dataStoreApp.editDataStoreKey(
                                    ITEMPACKING, it.description
                                )
                            }
                            viewModel.settingsItem[4].selected = it.description
                            viewModel.operatorUpdate(
                                workerId = mobileApplication.userId!!,
                                hashMapOf("itemPackingTypeFk" to it.description)
                            )
                            settingsAdapter!!.notifyItemChanged(4)

                            return@forEach
                        }
                    }
                }

                getString(R.string.selectSector) -> {
                    sectorListVO.forEach {
                        if (it.description == selected) {

                            runBlocking {
                                mobileApplication.dataStoreApp.editDataStoreKey(
                                    PRINTERNAME, getString(R.string.noprinter)
                                )
                                mobileApplication.dataStoreApp.editDataStoreKey(PRINTERFK, -1)

                                mobileApplication.dataStoreApp.editDataStoreKey(
                                    SECTORDESCRIP, it.description
                                )
                                mobileApplication.dataStoreApp.editDataStoreKey(
                                    SECTORFK, it.id
                                )
                                it.warehouseFk?.let { it1 ->
                                    mobileApplication.dataStoreApp.editDataStoreKey(
                                        WAREHOUSEFK, it1
                                    )
                                }
                            }

                            viewModel.settingsItem[0].sectorFk = it.id
                            viewModel.settingsItem[0].selected = it.description
                            viewModel.workerUpdateOperatorSalix(
                                "sector", mobileApplication.userId!!, it.id, null
                            )

                            settingsAdapter!!.notifyItemChanged(0)

                            return@forEach
                        }
                    }
                }

                getString(R.string.selectTrain) -> {
                    trainsList.forEach {
                        if (it.name == selected) {

                            runBlocking {
                                mobileApplication.dataStoreApp.editDataStoreKey(TRAINNAME, it.name)
                                mobileApplication.dataStoreApp.editDataStoreKey(TRAINFK, it.id)
                            }

                            viewModel.settingsItem[3].selected = it.name
                            viewModel.operatorUpdate(
                                workerId = mobileApplication.userId!!,
                                hashMapOf("trainFk" to it.id.toString())
                            )
                            settingsAdapter!!.notifyItemChanged(3)

                            return@forEach
                        }
                    }
                }

                getString(R.string.menuByDefault) -> {
                    array.forEach {
                        if (it == selected) {
                            runBlocking {
                                mobileApplication.dataStoreApp.editDataStoreKey(
                                    MENUBYDEFAULTSELECTED, if (it != getString(R.string.all)) {
                                        it
                                    } else {
                                        ""
                                    }
                                )
                            }
                            viewModel.settingsItem[6].selected = it
                            settingsAdapter!!.notifyItemChanged(6)

                            return@forEach

                        }
                    }
                }

                else -> {
                    printersList.forEach {
                        if (it.name == selected) {

                            runBlocking {
                                mobileApplication.dataStoreApp.editDataStoreKey(
                                    PRINTERNAME, it.name
                                )
                                mobileApplication.dataStoreApp.editDataStoreKey(PRINTERFK, it.id)
                            }

                            viewModel.settingsItem[2].selected = it.name
                            viewModel.workerUpdateOperatorSalix(
                                entity = "printer",
                                workerFk = mobileApplication.userId!!,
                                sectorFk = mobileApplication.dataStoreApp.readDataStoreKey(SECTORFK),
                                labelerFk = it.id
                            )
                            settingsAdapter!!.notifyItemChanged(2)

                            return@forEach
                        }
                    }
                }
            }

        }
        val dialog = builder.create()
        dialog.show()
    }

}