package es.verdnatura.presentation.view.feature.packaging.fragment.PackagingViewModel

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.map
import es.verdnatura.domain.SalixCallback
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.packaging.model.EntryList
import es.verdnatura.presentation.view.feature.packaging.model.EntrySalix
import es.verdnatura.presentation.view.feature.packaging.model.ItemSupplier
import es.verdnatura.presentation.view.feature.packaging.model.Supplier
import es.verdnatura.presentation.view.feature.packaging.model.SupplierList
import es.verdnatura.presentation.view.feature.packaging.model.itemSupplierList
import es.verdnatura.presentation.view.feature.packaging.model.notificationQueue
import es.verdnatura.presentation.view.feature.packaging.model.obervationEntry
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.Response
import java.io.File
import java.text.SimpleDateFormat
import java.util.Calendar
import java.util.Locale

class PackagingViewModel(val context: Context) : BaseViewModel(context) {

    private val _itemSupplierList by lazy { MutableLiveData<itemSupplierList>() }
    val itemSupplierList: LiveData<itemSupplierList>
        get() = _itemSupplierList

    private val _supplierList by lazy { MutableLiveData<SupplierList>() }
    val supplierList: LiveData<SupplierList>
        get() = _supplierList

    private val _entryList by lazy { MutableLiveData<EntryList>() }
    val entryList: LiveData<EntryList>
        get() = _entryList

    private val _entry by lazy { MutableLiveData<EntrySalix>() }
    val entry: LiveData<EntrySalix>
        get() = _entry

    private val _entryUpdate by lazy { MutableLiveData<Boolean>() }
    val entryUpdate: LiveData<Boolean>
        get() = _entryUpdate

    val loadEntryUpdate: LiveData<Event<Boolean>> = _entryUpdate.map { Event(it) }

    private val _entryAdd by lazy { MutableLiveData<EntrySalix>() }
    val entryAdd: LiveData<EntrySalix>
        get() = _entryAdd

    private val _uploadEntryImage by lazy { MutableLiveData<EntrySalix>() }
    val uploadEntryImage: LiveData<EntrySalix>
        get() = _uploadEntryImage

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    val loadEntryAdd: LiveData<Event<EntrySalix>> = _entryAdd.map { Event(it) }

    fun getSuppliers() {
        salix.getSuppliers()
            .enqueue(object :
                SalixCallback<List<Supplier>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<Supplier> = ArrayList()
                    listError.add(
                        Supplier(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _supplierList.value = SupplierList(listError)
                }

                override fun onSuccess(response: Response<List<Supplier>>) {
                    if (response.body() != null) {
                        _supplierList.value = response.body()?.let { SupplierList(it) }
                    } else {
                        val listError: ArrayList<Supplier> = ArrayList()
                        listError.add(
                            Supplier(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _supplierList.value = SupplierList(listError)

                    }
                }

            })
    }

    //http://localhost:3000/api/Entries/1/uploadFile?warehouseId=1&companyId=442&dmsTypeId=21&reference=1&description="Example description"&hasFile=false (tiene que ser de tipo Multipart y llevar adjunto un fichero)
    fun uploadFile(entry: EntrySalix) {
        salix.add_entry(entry)
            .enqueue(object :
                SalixCallback<EntrySalix>(context) {
                override fun onError(t: Throwable) {
                    _entry.value = EntrySalix(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<EntrySalix>) {
                    if (response.body() != null) {
                        _entry.value = response.body()
                    } else {
                        _entry.value = EntrySalix(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }

                }
            })
    }

    fun getEntriesFromSupplier(supplier: Int) {
        val calendar = Calendar.getInstance()
        calendar.add(Calendar.DAY_OF_YEAR, 0)

        val dateFormat = SimpleDateFormat("yyyy-MM-dd", Locale.getDefault())
        val today = dateFormat.format(calendar.time)

        salix.getEntriesFromSupplier("""{"where": {"supplierFk": "$supplier","created":{"gte":"$today"},"typeFk":"packaging","order":"created ASC"},"include": [ { "relation": "travel", "scope": { "fields": ["landed"]}}]}""")
            .enqueue(object :
                SalixCallback<List<EntrySalix>>(context) {

                override fun onSuccess(response: Response<List<EntrySalix>>) {
                    if (response.body() != null) {
                        _entryList.value = response.body()?.let { EntryList(it) }
                    } else {
                        val listError: ArrayList<EntrySalix> = ArrayList()
                        listError.add(
                            EntrySalix(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _entryList.value = EntryList(listError)

                    }
                }

            })
    }

    fun supplier_getItems(supplierFk: Any) {
        //pendiente de borrado si la tarea no se lleva a cabo
        /*  silex.supplier_getItems(supplierFk)
              .enqueue(object : SilexCallback<List<ItemSupplier>>(context) {
                  override fun onError(t: Throwable) {
                      val listError: ArrayList<ItemSupplier> = ArrayList()
                      listError.add(
                          ItemSupplier(
                              isError = true,
                              errorMessage = getMessageFromAllResponse(
                                  nameofFunction(this),
                                  t.message!!
                              )
                          )
                      )
                      _itemSupplierList.value = itemSupplierList(listError)
                  }

                  override fun onSuccess(response: Response<List<ItemSupplier>>) {
                      if (response.body() != null) {
                          _itemSupplierList.value =
                              response.body()?.let { itemSupplierList(it) }
                      } else {
                          val listError: ArrayList<ItemSupplier> = ArrayList()
                          listError.add(ItemSupplier(null, null))
                          _itemSupplierList.value = itemSupplierList(listError)
                      }
                  }

              })*/
    }

    fun entry_addSalix(entry: EntrySalix) {
        salix.add_entry(entry)
            .enqueue(object :
                SilexCallback<EntrySalix>(context) {
                override fun onError(t: Throwable) {
                    _entry.value = EntrySalix(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<EntrySalix>) {
                    if (response.body() != null) {
                        _entry.value = response.body()
                    } else {
                        _entry.value = EntrySalix(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }

                }
            })
    }

    fun entryUpdateObserv(entryId: Int, observation: String) {
        salix.entry_updateObserv(entryId, obervationEntry(observation))
            .enqueue(object :
                SalixCallback<EntrySalix>(context) {

                override fun onSuccess(response: Response<EntrySalix>) {
                    _entryUpdate.value = true
                }
            })
    }

    fun entry_uploadPhotoSalix(
        idEntry: Number,
        warehouseId: Number,
        companyId: Number,
        dmsTypeId: Number,
        reference: String,
        description: String,
        hasFile: Boolean,
        urlImage: String,

        ) {

        val file = File(urlImage)
        val fileRequestBody: RequestBody = RequestBody.create(MediaType.parse("image/jpeg"), file)

        salix.uploadEntryPhoto(
            idEntry,
            warehouseId,
            companyId,
            dmsTypeId,
            reference,
            description,
            hasFile,
            file = MultipartBody.Part.createFormData(
                "file", file.name, fileRequestBody
            )
        )
            .enqueue(object : SalixCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _uploadEntryImage.value = EntrySalix(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    super.onSuccess(response)
                }

            })
    }

    fun entry_addFromBuy(id: Number, item: Number, printedStickers: Number) {
        println("entryID -- addFromBuy")
        salix.addFromBuy(id, item, printedStickers)
            .enqueue(object : SalixCallback<EntrySalix>(context) {
                override fun onError(t: Throwable) {
                    _entryAdd.value = EntrySalix(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<EntrySalix>) {
                    if (response.body() != null) {
                        _entryAdd.value = response.body()
                    } else {
                        _entryAdd.value = EntrySalix(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }

                }
            })
    }

    fun notificationQueues(notification: String, workerfk: Int, bodyUrl: String) {
        salix.notificationQueues(notificationQueue(notification, workerfk, bodyUrl))
            .enqueue(object : SilexCallback<Void>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<Void>) {

                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            response.message()
                        )
                    )

                }
            })
    }

    fun entry_addFromPackaging(supplier: Int, isPackaging: Boolean) {
        salix.addFromPackaging(supplier, isPackaging)
            .enqueue(object : SalixCallback<EntrySalix>(context) {
                override fun onError(t: Throwable) {
                    _entryAdd.value = EntrySalix(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<EntrySalix>) {
                    if (response.body() != null) {
                        _entryAdd.value = response.body()
                    } else {
                        _entryAdd.value = EntrySalix(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }

                }
            })
    }

    fun getItemsPackaging(supplierId: Int, entryId: Int) {
        salix.getItemsPackaging(supplierId, entryId)
            .enqueue(object : SilexCallback<List<ItemSupplier>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<ItemSupplier> = ArrayList()
                    listError.add(
                        ItemSupplier(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _itemSupplierList.value = itemSupplierList(listError)
                }

                override fun onSuccess(response: Response<List<ItemSupplier>>) {
                    if (response.body() != null) {
                        _itemSupplierList.value =
                            response.body()?.let { itemSupplierList(it) }
                    } else {
                        val listError: ArrayList<ItemSupplier> = ArrayList()
                        listError.add(ItemSupplier(null, null))
                        _itemSupplierList.value = itemSupplierList(listError)
                    }
                }



            })
    }
}
