package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentItemexpeditionstateCardBinding
import es.verdnatura.domain.toInt
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionStateAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpedetionState
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionStateRow

@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class ExpeditionStateFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentItemexpeditionstateCardBinding, ExpeditionStateViewModel>(
    ExpeditionStateViewModel::class
) {

    private var adapter: ExpeditionStateAdapter? = null
    private var listItemsRow: ArrayList<ItemExpeditionStateRow> = ArrayList()
    private var itemScan = ""

    companion object {
        fun newInstance(entryPoint: String) = ExpeditionStateFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_itemexpeditionstate_card

    override fun init() {
        setEvents()
        setToolBar(entryPoint)
        super.init()
    }

    private fun setToolBar(title: String) {
        binding.mainToolbar.toolbarTitle.text = title
    }

    private fun setEvents() {
        binding.editItemFk.requestFocus()
        binding.editItemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (binding.editItemFk.text.toString().isNotEmpty()) {
                    try {
                        viewModel.expeditionGetStateJSON(binding.editItemFk.toInt())
                        itemScan = binding.editItemFk.text.toString()
                    } catch (ex: Exception) {
                        ma.messageWithSound(
                            message = getString(R.string.errorInput),
                            isError = true,
                            isToasted = true,
                            isPlayed = true,
                            titleWithError = "Error"
                        )
                    }

                }
                binding.editItemFk.setText("")
                ma.hideKeyboard(binding.editItemFk)
                return@setOnEditorActionListener true
            }
            false
        }


        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }
    }

    override fun observeViewModel() {
        with(viewModel) {
            itemExpedition.observe(viewLifecycleOwner) {
                setItemExpeditionState(it)
            }
        }
    }

    private fun setItemExpeditionState(itemInfo: ItemExpedetionState) {
        setToolBar(getString(R.string.expedition) + " " + itemScan)
        binding.itemcardLayout.visibility = View.VISIBLE

        listItemsRow = ArrayList()
        for (i in itemInfo.list) {
            listItemsRow.add(
                ItemExpeditionStateRow(
                    title = i.key,
                    value = i.value?.replace(".0", ""),
                    isEditable = false
                )
            )
        }
        adapter = ExpeditionStateAdapter(listItemsRow)
        binding.itemexpeditionstateRecyclerview.adapter = adapter
        binding.itemexpeditionstateRecyclerview.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

}

